@@ -28,25 +28,31 @@
 #define __CODECHAL_HW_H__
 
 #include "codechal.h"
-#include "renderhal.h"
+#include "renderhal_legacy.h"
 #include "mhw_mi.h"
-#include "mhw_render.h"
+#include "mhw_render_legacy.h"
 #include "mhw_state_heap.h"
 #include "mhw_vdbox.h"
 #include "mhw_vebox.h"
 #include "mhw_sfc.h"
 #include "mhw_cp_interface.h"
+#include "media_copy.h"
+#include "media_blt_copy.h"
 
 #include "mhw_vdbox_mfx_interface.h"
 #include "mhw_vdbox_hcp_interface.h"
 #include "mhw_vdbox_avp_interface.h"
 #include "mhw_vdbox_huc_interface.h"
 #include "mhw_vdbox_vdenc_interface.h"
+#include "mhw_vdbox_hcp_itf.h"
 
 #include "media_interfaces_mhw.h"
+#include "media_sfc_interface.h"
 
 #include "gfxmacro.h"
 
+#include "codec_hw_next.h"
+
 //------------------------------------------------------------------------------
 // Macros specific to MOS_CODEC_SUBCOMP_HW sub-comp
 //------------------------------------------------------------------------------
@@ -86,10 +92,6 @@
 #define CODECHAL_HW_CHK_COND_RETURN(_expr, _message, ...)                           \
     MOS_CHK_COND_RETURN(MOS_COMPONENT_CODEC, MOS_CODEC_SUBCOMP_HW,_expr,_message, ##__VA_ARGS__)
 
-#define CODECHAL_CACHELINE_SIZE                 64
-#define CODECHAL_PAGE_SIZE                      0x1000
-#define CODECHAL_PAK_OBJ_EACH_CU                66
-
 #define CODECHAL_SURFACE_PITCH_ALIGNMENT        128
 
 #define CODECHAL_MEDIA_WALKER_MAX_COLORS        16      // 4 Bits for color field gives max 16 colors
@@ -102,9 +104,6 @@
 #define CODECHAL_VLINESTRIDEOFFSET_TOP_FIELD    0
 #define CODECHAL_VLINESTRIDEOFFSET_BOT_FIELD    1
 
-// Params for Huc
-#define HUC_DMEM_OFFSET_RTOS_GEMS                       0x2000
-
 #define CODECHAL_MAX_DEPENDENCY_COUNT  8
 
 #define CODECHAL_INVALID_BINDING_TABLE_IDX  0xFFFFFFFF
@@ -146,73 +145,6 @@ enum CodechalWalkingPattern
     codechalVerticalRasterScan               = 4
 };
 
-typedef enum _CODECHAL_MEDIA_STATE_TYPE
-{
-    CODECHAL_MEDIA_STATE_OLP                                = 0,
-    CODECHAL_MEDIA_STATE_ENC_NORMAL                         = 1,
-    CODECHAL_MEDIA_STATE_ENC_PERFORMANCE                    = 2,
-    CODECHAL_MEDIA_STATE_ENC_QUALITY                        = 3,
-    CODECHAL_MEDIA_STATE_ENC_I_FRAME_DIST                   = 4,
-    CODECHAL_MEDIA_STATE_32X_SCALING                        = 5,
-    CODECHAL_MEDIA_STATE_16X_SCALING                        = 6,
-    CODECHAL_MEDIA_STATE_4X_SCALING                         = 7,
-    CODECHAL_MEDIA_STATE_32X_ME                             = 8,
-    CODECHAL_MEDIA_STATE_16X_ME                             = 9,
-    CODECHAL_MEDIA_STATE_4X_ME                              = 10,
-    CODECHAL_MEDIA_STATE_BRC_INIT_RESET                     = 11,
-    CODECHAL_MEDIA_STATE_BRC_UPDATE                         = 12,
-    CODECHAL_MEDIA_STATE_BRC_BLOCK_COPY                     = 13,
-    CODECHAL_MEDIA_STATE_HYBRID_PAK_P1                      = 14,
-    CODECHAL_MEDIA_STATE_HYBRID_PAK_P2                      = 15,
-    CODECHAL_MEDIA_STATE_ENC_I_FRAME_CHROMA                 = 16,
-    CODECHAL_MEDIA_STATE_ENC_I_FRAME_LUMA                   = 17,
-    CODECHAL_MEDIA_STATE_MPU_FHB                            = 18,
-    CODECHAL_MEDIA_STATE_TPU_FHB                            = 19,
-    CODECHAL_MEDIA_STATE_PA_COPY                            = 20,
-    CODECHAL_MEDIA_STATE_PL2_COPY                           = 21,
-    CODECHAL_MEDIA_STATE_ENC_ADV                            = 22,
-    CODECHAL_MEDIA_STATE_2X_SCALING                         = 23,
-    CODECHAL_MEDIA_STATE_32x32_PU_MODE_DECISION             = 24,
-    CODECHAL_MEDIA_STATE_16x16_PU_SAD                       = 25,
-    CODECHAL_MEDIA_STATE_16x16_PU_MODE_DECISION             = 26,
-    CODECHAL_MEDIA_STATE_8x8_PU                             = 27,
-    CODECHAL_MEDIA_STATE_8x8_PU_FMODE                       = 28,
-    CODECHAL_MEDIA_STATE_32x32_B_INTRA_CHECK                = 29,
-    CODECHAL_MEDIA_STATE_HEVC_B_MBENC                       = 30,
-    CODECHAL_MEDIA_STATE_RESET_VLINE_STRIDE                 = 31,
-    CODECHAL_MEDIA_STATE_HEVC_B_PAK                         = 32,
-    CODECHAL_MEDIA_STATE_HEVC_BRC_LCU_UPDATE                = 33,
-    CODECHAL_MEDIA_STATE_ME_VDENC_STREAMIN                  = 34,
-    CODECHAL_MEDIA_STATE_VP9_ENC_I_32x32                    = 35,
-    CODECHAL_MEDIA_STATE_VP9_ENC_I_16x16                    = 36,
-    CODECHAL_MEDIA_STATE_VP9_ENC_P                          = 37,
-    CODECHAL_MEDIA_STATE_VP9_ENC_TX                         = 38,
-    CODECHAL_MEDIA_STATE_VP9_DYS                            = 39,
-    CODECHAL_MEDIA_STATE_VP9_PAK_LUMA_RECON                 = 40,
-    CODECHAL_MEDIA_STATE_VP9_PAK_CHROMA_RECON               = 41,
-    CODECHAL_MEDIA_STATE_VP9_PAK_DEBLOCK_MASK               = 42,
-    CODECHAL_MEDIA_STATE_VP9_PAK_LUMA_DEBLOCK               = 43,
-    CODECHAL_MEDIA_STATE_VP9_PAK_CHROMA_DEBLOCK             = 44,
-    CODECHAL_MEDIA_STATE_VP9_PAK_MC_PRED                    = 45,
-    CODECHAL_MEDIA_STATE_VP9_PAK_P_FRAME_LUMA_RECON         = 46,
-    CODECHAL_MEDIA_STATE_VP9_PAK_P_FRAME_LUMA_RECON_32x32   = 47,
-    CODECHAL_MEDIA_STATE_VP9_PAK_P_FRAME_CHROMA_RECON       = 48,
-    CODECHAL_MEDIA_STATE_VP9_PAK_P_FRAME_INTRA_LUMA_RECON   = 49,
-    CODECHAL_MEDIA_STATE_VP9_PAK_P_FRAME_INTRA_CHROMA_RECON = 50,
-    CODECHAL_MEDIA_STATE_PREPROC                            = 51,
-    CODECHAL_MEDIA_STATE_ENC_WP                             = 52,
-    CODECHAL_MEDIA_STATE_HEVC_I_MBENC                       = 53,
-    CODECHAL_MEDIA_STATE_CSC_DS_COPY                        = 54,
-    CODECHAL_MEDIA_STATE_2X_4X_SCALING                      = 55,
-    CODECHAL_MEDIA_STATE_HEVC_LCU64_B_MBENC                 = 56,
-    CODECHAL_MEDIA_STATE_MB_BRC_UPDATE                      = 57,
-    CODECHAL_MEDIA_STATE_STATIC_FRAME_DETECTION             = 58,
-    CODECHAL_MEDIA_STATE_HEVC_ROI                           = 59,
-    CODECHAL_MEDIA_STATE_SW_SCOREBOARD_INIT                 = 60,
-    CODECHAL_NUM_MEDIA_STATES                               = 61
-} CODECHAL_MEDIA_STATE_TYPE;
-C_ASSERT(CODECHAL_NUM_MEDIA_STATES == CODECHAL_MEDIA_STATE_SW_SCOREBOARD_INIT + 1);  //!< update this and add new entry in the default SSEU table for each platform()
-
 typedef enum _CODECHAL_SLICE_STATE
 {
     CODECHAL_SLICE_SHUTDOWN_DEFAULT     = 0,
@@ -251,36 +183,6 @@ struct CodechalQpStatusCount
     };
 };
 
-//!
-//! \struct    CodechalHucStreamoutParams
-//! \brief     Codechal Huc streamout parameters
-//!
-struct CodechalHucStreamoutParams
-{
-    CODECHAL_MODE       mode;
-
-    // Indirect object addr command params
-    PMOS_RESOURCE       dataBuffer;
-    uint32_t            dataSize;              // 4k aligned
-    uint32_t            dataOffset;            // 4k aligned
-    PMOS_RESOURCE       streamOutObjectBuffer;
-    uint32_t            streamOutObjectSize;   // 4k aligned
-    uint32_t            streamOutObjectOffset; //4k aligned
-
-    // Stream object params
-    uint32_t            indStreamInLength;
-    uint32_t            inputRelativeOffset;
-    uint32_t            outputRelativeOffset;
-
-    // Segment Info
-    void               *segmentInfo;
-
-    // Indirect Security State
-    MOS_RESOURCE        hucIndState;
-    uint32_t            curIndEntriesNum;
-    uint32_t            curNumSegments;
-};
-
 //!
 //! \struct    CodechalDataCopyParams
 //! \brief     Codechal data copy parameters
@@ -298,38 +200,6 @@ struct CodechalDataCopyParams
     uint32_t        dstOffset;
 };
 
-//!
-//! \struct    EncodeStatusReadParams
-//! \brief     Read encode states parameters
-//!
-struct EncodeStatusReadParams
-{
-    bool          vdencBrcEnabled;
-    bool          waReadVDEncOverflowStatus;
-    uint32_t      mode ;
-
-    uint32_t      vdencBrcNumOfSliceOffset;
-    PMOS_RESOURCE *resVdencBrcUpdateDmemBufferPtr;
-
-    PMOS_RESOURCE resBitstreamByteCountPerFrame;
-    uint32_t      bitstreamByteCountPerFrameOffset;
-
-    PMOS_RESOURCE resBitstreamSyntaxElementOnlyBitCount;
-    uint32_t      bitstreamSyntaxElementOnlyBitCountOffset;
-
-    PMOS_RESOURCE resQpStatusCount;
-    uint32_t      qpStatusCountOffset;
-
-    PMOS_RESOURCE resNumSlices;
-    uint32_t      numSlicesOffset;
-
-    PMOS_RESOURCE resImageStatusMask;
-    uint32_t      imageStatusMaskOffset;
-
-    PMOS_RESOURCE resImageStatusCtrl;
-    uint32_t      imageStatusCtrlOffset;
-};
-
 //!
 //! \struct    BrcPakStatsReadParams
 //! \brief     Read brc pak states parameters
@@ -380,6 +250,10 @@ protected:
     MhwVdboxAvpInterface            *m_avpInterface = nullptr;        //!< Pointer to Mhw avp interface
     MhwVdboxHucInterface            *m_hucInterface = nullptr;        //!< Pointer to Mhw huc interface
     MhwVdboxVdencInterface          *m_vdencInterface = nullptr;      //!< Pointer to Mhw vdenc interface
+    std::shared_ptr<mhw::vdbox::hcp::Itf>   m_hcpItf   = nullptr;
+    std::shared_ptr<mhw::vdbox::vdenc::Itf> m_vdencItf = nullptr;
+    std::shared_ptr<MediaSfcInterface> m_mediaSfcItf = nullptr;
+    std::shared_ptr<mhw::render::Itf>       m_renderItf             = nullptr;
 
     CODECHAL_SSEU_SETTING const         *m_ssEuTable = nullptr;       //!< Pointer to the default SSEU settings table
     uint16_t                            m_numMediaStates = CODECHAL_NUM_MEDIA_STATES;  //!< number of media states
@@ -458,6 +332,135 @@ public:
         MhwInterfaces     *mhwInterfaces,
         bool              disableScalability = false);
 
+    static CodechalHwInterface *Create(
+        PMOS_INTERFACE    osInterface,
+        CODECHAL_FUNCTION codecFunction,
+        MhwInterfaces     *mhwInterfaces,
+        bool              disableScalability);
+
+    //!
+    //! \brief    Get avp interface
+    //! \details  Get avp interface in codechal hw interface next
+    //!
+    //! \return    pointer to new AVP interface
+    //!
+    inline std::shared_ptr<mhw::vdbox::avp::Itf> GetAvpInterfaceNext()
+    {
+        if (m_hwInterfaceNext)
+        {
+            return m_hwInterfaceNext->GetAvpInterfaceNext();
+        }
+        else
+        {
+            return nullptr;
+        }
+    }
+
+    //!
+    //! \brief    Get vdenc interface
+    //! \details  Get vdenc interface in codechal hw interface next
+    //!
+    //! \return    pointer to new Vdenc interface
+    //!
+    inline std::shared_ptr<mhw::vdbox::vdenc::Itf> GetVdencInterfaceNext()
+    {
+        if (m_hwInterfaceNext)
+        {
+            return m_hwInterfaceNext->GetVdencInterfaceNext();
+        }
+        else
+        {
+            return nullptr;
+        }
+    }
+
+    //! \brief    Get huc interface
+    //! \details  Get huc interface in codechal hw interface next
+    //!
+    //! \return    pointer to new HUC interface
+    //!
+    inline std::shared_ptr<mhw::vdbox::huc::Itf> GetHucInterfaceNext()
+    {
+        if (m_hwInterfaceNext)
+        {
+            return m_hwInterfaceNext->GetHucInterfaceNext();
+        }
+        else
+        {
+            return nullptr;
+        }
+    }
+
+    //!
+    //! \brief    Get mi interface
+    //! \details  Get mi interface in codechal hw interface next
+    //!
+    //! \return    pointer to new MI interface
+    //!
+    inline std::shared_ptr<mhw::mi::Itf> GetMiInterfaceNext()
+    {
+        if (m_hwInterfaceNext)
+        {
+            return m_hwInterfaceNext->GetMiInterfaceNext();
+        }
+        else
+        {
+            return nullptr;
+        }
+    }
+
+    //!
+    //! \brief    Get hcp interface
+    //! \details  Get hcp interface in codechal hw interface next
+    //!
+    //! \return    pointer to new HCP interface
+    //!
+    inline std::shared_ptr<mhw::vdbox::hcp::Itf> GetHcpInterfaceNext()
+    {
+        if (m_hwInterfaceNext)
+        {
+            return m_hwInterfaceNext->GetHcpInterfaceNext();
+        }
+        else
+        {
+            return nullptr;
+        }
+    }
+
+    //!
+    //! \brief    Get mfx interface
+    //! \details  Get mfx interface in codechal hw interface next
+    //!
+    //! \return    pointer to new MFX interface
+    //!
+    inline std::shared_ptr<mhw::vdbox::mfx::Itf> GetMfxInterfaceNext()
+    {
+        if (m_hwInterfaceNext)
+        {
+            return m_hwInterfaceNext->GetMfxInterfaceNext();
+        }
+        else
+        {
+            return nullptr;
+        }
+    }
+    //!
+    //! \brief    Get render interface
+    //! \details  Get render interface in codechal hw interface next
+    //!
+    //! \return    pointer to new HCP interface
+    //!
+    inline std::shared_ptr<mhw::render::Itf> GetRenderInterfaceNext()
+    {
+        if (m_hwInterfaceNext)
+        {
+            return m_hwInterfaceNext->GetRenderInterfaceNext();
+        }
+        else
+        {
+            return nullptr;
+        }
+    }
     //!
     //! \brief    Copy constructor
     //!
@@ -539,7 +542,15 @@ public:
             m_sfcInterface = nullptr;
         }
 
-        MHW_HWCMDPARSER_DESTROY();
+        if (m_hwInterfaceNext)
+        {
+            MOS_Delete(m_hwInterfaceNext);
+            m_hwInterfaceNext = nullptr;
+        }
+
+    #if MHW_HWCMDPARSER_ENABLED
+        mhw::HwcmdParser::DestroyInstance();
+    #endif
     }
 
     //!
@@ -554,6 +565,11 @@ public:
         return m_osInterface;
     }
 
+    inline std::shared_ptr<MediaSfcInterface> GetMediaSfcInterface()
+    {
+        return m_mediaSfcItf;
+    }
+
     //!
     //! \brief    Get State Heap Settings
     //! \details  Get State Heap Settings in codechal hw interface 
@@ -746,6 +762,29 @@ public:
         return m_cacheabilitySettings;
     }
 
+    //! \brief    Create media copy
+    //! \details  Create media copy instance.
+    //! \param    osInterface
+    //!           [in] Pointer to MOS_INTERFACE.
+    //! \return   MediaCopyBaseState*
+    //!           Pointer to MediaCopyBaseState
+    //!
+    virtual MediaCopyBaseState* CreateMediaCopy(PMOS_INTERFACE mosInterface)
+    {
+        return nullptr;
+    }
+
+    //! \brief    Get blt state
+    //! \details  Get blt interface in codechal hw interface
+    //!
+    //! \return   [out] BltState*
+    //!           Interface got.
+    //!
+    virtual BltState *GetBltState()
+    {
+        return nullptr;
+    }
+
     //!
     //! \brief    Set Cacheability Settings
     //! \details  Set Cacheability Settings in sub interfaces in codechal hw interface 
@@ -1029,6 +1068,17 @@ public:
         uint32_t                       *commandsSize,
         uint32_t                       *patchListSize);
 
+    //!
+    //! \brief    Get max vdbox index
+    //!
+    //! \return   MHW_VDBOX_NODE_IND
+    //!           max vdbox index got
+    //!
+    inline MHW_VDBOX_NODE_IND GetMaxVdboxIndex()
+    {
+        return MEDIA_IS_SKU(m_skuTable, FtrVcs2) ? MHW_VDBOX_NODE_2 : MHW_VDBOX_NODE_1;
+    }
+
     //!
     //! \brief    Initialize the codechal hw interface
     //! \details  Initialize the interface before using
@@ -1657,6 +1707,23 @@ public:
     
     //! \brief    default disable the get vdbox node by UMD, decided by MHW and MOS
     bool m_getVdboxNodeByUMD = false;
+
+    operator CodechalHwInterfaceNext &()
+    {
+        if (m_hwInterfaceNext == nullptr)
+        {
+            CODECHAL_HW_ASSERTMESSAGE("Conversion cannot succeed due to null pointer of m_hwInterfaceNext.");
+        }
+        return *m_hwInterfaceNext;
+    }
+
+    operator CodechalHwInterfaceNext *()
+    {
+        return m_hwInterfaceNext;
+    }
+
+    CodechalHwInterfaceNext *m_hwInterfaceNext = nullptr;
+
 };
 
 extern const MOS_SYNC_PARAMS                        g_cInitSyncParams;
