@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2018-2020, Intel Corporation
+* Copyright (c) 2018-2023, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,8 @@
 #include "linux_skuwa_debug.h"
 #include "linux_media_skuwa.h"
 #include "mos_utilities.h"
+#include "mos_os_specific.h"
+#include "media_user_setting_specific.h"
 
 static constexpr uint32_t singleVeboxSubSliceNumMax = 24;
 
@@ -63,7 +65,8 @@ static struct LinuxCodecInfo tglCodecInfo =
 
 static bool InitTglMediaSku(struct GfxDeviceInfo *devInfo,
                              MediaFeatureTable *skuTable,
-                             struct LinuxDriverInfo *drvInfo)
+                             struct LinuxDriverInfo *drvInfo,
+                             MediaUserSettingSharedPtr userSettingPtr)
 {
     if ((devInfo == nullptr) || (skuTable == nullptr) || (drvInfo == nullptr))
     {
@@ -135,8 +138,8 @@ static bool InitTglMediaSku(struct GfxDeviceInfo *devInfo,
 
         /* HEVC VDENC Main8/10 422/444 Encoding. */
         MEDIA_WR_SKU(skuTable, FtrEncodeHEVCVdencMain444, codecInfo->hevcVdenc);
-        MEDIA_WR_SKU(skuTable, FtrEncodeHEVCVdencMain10, codecInfo->hevcVdenc);
-        MEDIA_WR_SKU(skuTable, FtrEncodeHEVCVdencMain10bit422, codecInfo->hevcVdenc);
+        MEDIA_WR_SKU(skuTable, FtrEncodeHEVCVdencMain422, 0);
+        MEDIA_WR_SKU(skuTable, FtrEncodeHEVCVdencMain10bit422, 0);
         MEDIA_WR_SKU(skuTable, FtrEncodeHEVCVdencMain10bit444, codecInfo->hevcVdenc);
 
         /* HEVC VDENC Main8/10bit-420/422/444 Scc Encoding. */
@@ -225,18 +228,6 @@ static bool InitTglMediaSku(struct GfxDeviceInfo *devInfo,
 
     MEDIA_WR_SKU(skuTable, FtrContextBasedScheduling, 1);
     MEDIA_WR_SKU(skuTable, FtrSfcScalability, 1);
-    // Enable context based scheduling when vebox scalabilitiy enable due to KMD didn't support GUC submission yet
-    MOS_USER_FEATURE_VALUE_DATA userFeatureData;
-    MOS_ZeroMemory(&userFeatureData, sizeof(userFeatureData));
-    MOS_UserFeature_ReadValue_ID(
-        nullptr,
-        __MEDIA_USER_FEATURE_VALUE_ENABLE_VEBOX_SCALABILITY_MODE_ID,
-        &userFeatureData,
-        nullptr);
-    if (userFeatureData.i32Data)
-    {
-        MEDIA_WR_SKU(skuTable, FtrContextBasedScheduling, 1);
-    }
 
     MEDIA_WR_SKU(skuTable, FtrTileY, 1);
 
@@ -250,12 +241,13 @@ static bool InitTglMediaSku(struct GfxDeviceInfo *devInfo,
     }
 
     // Disable MMC for all components if set reg key
+    MOS_USER_FEATURE_VALUE_DATA userFeatureData;
     MOS_ZeroMemory(&userFeatureData, sizeof(userFeatureData));
     MOS_UserFeature_ReadValue_ID(
         nullptr,
         __MEDIA_USER_FEATURE_VALUE_DISABLE_MMC_ID,
         &userFeatureData,
-        nullptr);
+        (MOS_CONTEXT_HANDLE)nullptr);
     if (userFeatureData.bData)
     {
         disableMMC = true;
@@ -272,7 +264,7 @@ static bool InitTglMediaSku(struct GfxDeviceInfo *devInfo,
         nullptr,
         __MEDIA_USER_FEATURE_VALUE_FORCE_MMC_ON_ID,
         &userFeatureData,
-        nullptr);
+        (MOS_CONTEXT_HANDLE)nullptr);
 
     if (userFeatureData.bData)
     {
@@ -282,6 +274,26 @@ static bool InitTglMediaSku(struct GfxDeviceInfo *devInfo,
     MEDIA_WR_SKU(skuTable, FtrLinearCCS, 1);
 
     MEDIA_WR_SKU(skuTable, FtrUseSwSwizzling, 1);
+    MEDIA_WR_SKU(skuTable, FtrScalingFirst, 1);
+
+    // Create uncompressible surface by default
+    MEDIA_WR_SKU(skuTable, FtrCompressibleSurfaceDefault, 0);
+
+    bool compressibleSurfaceEnable = false;
+
+    ReadUserSetting(userSettingPtr,
+        compressibleSurfaceEnable,
+        "Enable Compressible Surface Creation",
+        MediaUserSetting::Group::Device);
+        
+#ifdef _MMC_SUPPORTED
+    if (compressibleSurfaceEnable)
+    {
+        MEDIA_WR_SKU(skuTable, FtrCompressibleSurfaceDefault, 1);
+    }
+#endif
+
+    MEDIA_WR_SKU(skuTable, FtrHDR, 1);
 
     return true;
 }
@@ -312,7 +324,7 @@ static bool InitTglMediaWa(struct GfxDeviceInfo *devInfo,
         nullptr,
         __MEDIA_USER_FEATURE_VALUE_AUX_TABLE_16K_GRANULAR_ID,
         &userFeatureData,
-        nullptr);
+        (MOS_CONTEXT_HANDLE)nullptr);
 
     MEDIA_WR_WA(waTable, WaDummyReference, 1);
 
@@ -352,16 +364,19 @@ static bool InitTglMediaWa(struct GfxDeviceInfo *devInfo,
     /*software wa to use huc copy for init aux table for MFX*/
     MEDIA_WR_WA(waTable, Wa_22010493002, 1);
 
+    MEDIA_WR_WA(waTable, Wa_Vp9UnalignedHeight, 1);
+    MEDIA_WR_WA(waTable, WaDisableSetObjectCapture, 0);
     return true;
 }
 
 
 #ifdef IGFX_GEN12_DG1_SUPPORTED
 static bool InitDG1MediaSku(struct GfxDeviceInfo *devInfo,
-                              MediaFeatureTable *skuTable,
-                           struct LinuxDriverInfo *drvInfo)
+                            MediaFeatureTable *skuTable,
+                            struct LinuxDriverInfo *drvInfo,
+                            MediaUserSettingSharedPtr userSettingPtr)
 {
-    if (!InitTglMediaSku(devInfo, skuTable, drvInfo))
+    if (!InitTglMediaSku(devInfo, skuTable, drvInfo, userSettingPtr))
     {
         return false;
     }
@@ -383,7 +398,7 @@ static bool InitDG1MediaSku(struct GfxDeviceInfo *devInfo,
         nullptr,
         __VPHAL_ENABLE_MMC_ID,
         &userFeatureData,
-        nullptr);
+        (MOS_CONTEXT_HANDLE)nullptr);
     if (userFeatureData.bData)
     {
         enableVPMMC = true;
@@ -394,7 +409,7 @@ static bool InitDG1MediaSku(struct GfxDeviceInfo *devInfo,
         nullptr,
         __MEDIA_USER_FEATURE_VALUE_CODEC_MMC_ENABLE_ID,
         &userFeatureData,
-        nullptr);
+        (MOS_CONTEXT_HANDLE)nullptr);
     if (userFeatureData.bData)
     {
         enableCodecMMC = true;
@@ -444,9 +459,10 @@ static bool dg1DeviceRegister = DeviceInfoFactory<LinuxDeviceInit>::
 #ifdef IGFX_GEN12_RKL_SUPPORTED
 static bool InitRKLMediaSku(struct GfxDeviceInfo *devInfo,
     MediaFeatureTable *                           skuTable,
-    struct LinuxDriverInfo *                      drvInfo)
+    struct LinuxDriverInfo                       *drvInfo,
+    MediaUserSettingSharedPtr                     userSettingPtr)
 {
-    if (!InitTglMediaSku(devInfo, skuTable, drvInfo))
+    if (!InitTglMediaSku(devInfo, skuTable, drvInfo, userSettingPtr))
     {
         return false;
     }
@@ -459,6 +475,8 @@ static bool InitRKLMediaSku(struct GfxDeviceInfo *devInfo,
     MEDIA_WR_SKU(skuTable, FtrSfcScalability, 0);
     MEDIA_WR_SKU(skuTable, FtrIntelVP8VLDDecoding, 0);
 
+    // turn off Compression on RKL as HW known issue
+    MEDIA_WR_SKU(skuTable, FtrE2ECompression, 0);
     return true;
 }
 
@@ -489,9 +507,10 @@ static bool rklDeviceRegister = DeviceInfoFactory<LinuxDeviceInit>::
 #ifdef IGFX_GEN12_ADLS_SUPPORTED
 static bool InitAdlsMediaSku(struct GfxDeviceInfo *devInfo,
                              MediaFeatureTable *skuTable,
-                             struct LinuxDriverInfo *drvInfo)
+                             struct LinuxDriverInfo *drvInfo,
+                             MediaUserSettingSharedPtr userSettingPtr)
 {
-    if (!InitTglMediaSku(devInfo, skuTable, drvInfo))
+    if (!InitTglMediaSku(devInfo, skuTable, drvInfo, userSettingPtr))
     {
         return false;
     }
@@ -506,6 +525,10 @@ static bool InitAdlsMediaSku(struct GfxDeviceInfo *devInfo,
     //Disable VP8 for ADLS
     MEDIA_WR_SKU(skuTable, FtrIntelVP8VLDDecoding, 0);
 
+    // Disable HEVC 422 Virtual Tile Scalability
+    MEDIA_WR_SKU(skuTable, FtrDecodeHEVC422VTScalaDisable, 1);
+    MEDIA_WR_SKU(skuTable, FtrE2ECompression, 1);
+
     return true;
 }
 
@@ -521,6 +544,11 @@ static bool InitAdlsMediaWa(struct GfxDeviceInfo* devInfo,
     //ADL-S not need this
     MEDIA_WR_WA(waTable, Wa_1409820462, 0);
 
+    //ADL-S need enable MMC for all stepping
+    MEDIA_WR_WA(waTable, WaDisableCodecMmc, 0);
+    MEDIA_WR_WA(waTable, WaDisableVPMmc, 0);
+    MEDIA_WR_WA(waTable, WaDisableClearCCS, 1);
+
     return true;
 }
 
@@ -539,9 +567,10 @@ static bool adlsDeviceRegister = DeviceInfoFactory<LinuxDeviceInit>::
 #ifdef IGFX_GEN12_ADLP_SUPPORTED
 static bool InitAdlpMediaSku(struct GfxDeviceInfo *devInfo,
     MediaFeatureTable *                            skuTable,
-    struct LinuxDriverInfo *                       drvInfo)
+    struct LinuxDriverInfo                        *drvInfo,
+    MediaUserSettingSharedPtr                      userSettingPtr)
 {
-    if (!InitTglMediaSku(devInfo, skuTable, drvInfo))
+    if (!InitTglMediaSku(devInfo, skuTable, drvInfo, userSettingPtr))
     {
         return false;
     }
@@ -552,6 +581,8 @@ static bool InitAdlpMediaSku(struct GfxDeviceInfo *devInfo,
     }
 
     MEDIA_WR_SKU(skuTable, FtrAV1VLDLSTDecoding, 1);
+    MEDIA_WR_SKU(skuTable, FtrGucSubmission, 1);
+    MEDIA_WR_SKU(skuTable, FtrDecodeHEVC422VTScalaDisable, 1);
 
     return true;
 }
@@ -567,7 +598,7 @@ static bool InitAdlpMediaWa(struct GfxDeviceInfo *devInfo,
 
     //ADL-P not need this
     MEDIA_WR_WA(waTable, Wa_1409820462, 0);
-
+    MEDIA_WR_WA(waTable, WaDisableClearCCS, 1);
     return true;
 }
 
@@ -583,6 +614,61 @@ static bool adlpDeviceRegister = DeviceInfoFactory<LinuxDeviceInit>::
 
 #endif
 
+#ifdef IGFX_GEN12_ADLN_SUPPORTED
+static bool InitAdlnMediaSku(struct GfxDeviceInfo *devInfo,
+    MediaFeatureTable *                            skuTable,
+    struct LinuxDriverInfo                        *drvInfo,
+    MediaUserSettingSharedPtr                      userSettingPtr)
+{
+    if (!InitTglMediaSku(devInfo, skuTable, drvInfo, userSettingPtr))
+    {
+        return false;
+    }
+
+    if (devInfo->eGTType == GTTYPE_GT0_5)
+    {
+        MEDIA_WR_SKU(skuTable, FtrGT0_5, 1);
+    }
+
+    MEDIA_WR_SKU(skuTable, FtrAV1VLDLSTDecoding, 1);
+    MEDIA_WR_SKU(skuTable, FtrGucSubmission, 1);
+    MEDIA_WR_SKU(skuTable, FtrDecodeHEVC422VTScalaDisable, 1);
+    MEDIA_WR_SKU(skuTable, FtrE2ECompression, 1);
+
+    return true;
+}
+
+static bool InitAdlnMediaWa(struct GfxDeviceInfo *devInfo,
+    MediaWaTable *                                waTable,
+    struct LinuxDriverInfo *                      drvInfo)
+{
+    if (!InitTglMediaWa(devInfo, waTable, drvInfo))
+    {
+        return false;
+    }
+
+    //ADL-N keeps below setting same as ADL-S/ADL-P
+    MEDIA_WR_WA(waTable, Wa_1409820462, 0);
+
+    //ADL-N need enable MMC for all stepping
+    MEDIA_WR_WA(waTable, WaDisableCodecMmc, 0);
+    MEDIA_WR_WA(waTable, WaDisableVPMmc, 0);
+    MEDIA_WR_WA(waTable, WaDisableClearCCS, 1);
+
+    return true;
+}
+
+static struct LinuxDeviceInit adlnDeviceInit =
+    {
+        .productFamily    = IGFX_ALDERLAKE_N,
+        .InitMediaFeature = InitAdlnMediaSku,
+        .InitMediaWa      = InitAdlnMediaWa,
+};
+
+static bool adlnDeviceRegister = DeviceInfoFactory<LinuxDeviceInit>::
+    RegisterDevice(IGFX_ALDERLAKE_N, &adlnDeviceInit);
+
+#endif
 
 static struct LinuxDeviceInit tgllpDeviceInit =
 {
